from logging import raiseExceptions
from tokenize import String
from matplotlib.cbook import boxplot_stats
from PAPyA.config_loader import Loader
from PAPyA.file_reader import FileReader
from PAPyA.get_ranks import getRanks
from PAPyA.nsga2 import Nsga2
import yaml
import warnings
import itertools
import plotly.express as px
import pandas as pd
from itertools import islice
import matplotlib.pyplot as plt
import scipy.stats as ss
import numpy as np
from numpy.ma.core import maximum_fill_value
import seaborn as sns
import matplotlib.pyplot as plt
import math

# SINGEL DIMENSION

class SDRank(FileReader):
    def __init__(self, config_path: str, log_path: str, size: str, sd: str):
        super().__init__(config_path, log_path, size, sd)
       
    # ON DEVELOPMENT 
    
    def replicability_plot(self,dimension_view: str, mode=0):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get("dimensions")
        choosen_dimension = self.sd
        
        for key,value in d.items():
            if key == choosen_dimension:
                option_list = value
            if key == dimension_view:
                view_list = value
            
        full_data = []
        for x in option_list:
            table = self.replicability(x, mode)
            for i in view_list:
                replicability_score = table.loc[i].item()
                full_data.append(replicability_score)

        arr1 = [full_data[0], full_data[2], full_data[4]]
        arr2 = [full_data[1], full_data[3], full_data[5]]
        total_arr = []
        for x in range(len(view_list)):
            arr =  []
            counter = x
            for i in option_list:
                arr.append(full_data[counter])
                counter = counter + len(view_list)  
            plt.plot(option_list, arr, label=view_list[x])
        plt.legend()
        plt.show()
        
    def replicability(self, options: str, mode = 0):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get("dimensions")
        q = data.get("query")
        choosen_dimension = self.sd 
        
        if options in d[choosen_dimension]:
            if mode == 1:
                dimensions = []
                for key,value in d.items():
                    if key != choosen_dimension:
                        dimensions.append(key)
                
                total_result = []
                for x in dimensions:
                    scores_per_dimension = []
                    idx = []
                    for i in d[x]:
                        self.sd = x
                        df = self.calculateRank(i, options)["Result"]
                        idx.append(i)
                        df = df.to_list()
                        replicability_score = sum(df)/len(df)
                        scores_per_dimension.append(replicability_score)
                    result = pd.DataFrame(scores_per_dimension, columns=[options], index = idx)
                    total_result.append(result)
                table = pd.concat(total_result)
                self.sd = choosen_dimension
                return table
                
            elif mode == 0:
                dimensions = []
                for key,value in d.items():
                    if key != choosen_dimension:
                        dimensions.append(key)
                
                total_result = []
                for x in dimensions:
                    scores_per_dimension = []
                    idx = []
                    for i in d[x]:
                        self.sd = x
                        df = self.calculateRank(i, options)["Rank 1"]
                        idx.append(i)
                        df = df.to_list()
                        replicability_score = sum(df)/(q*len(df))
                        scores_per_dimension.append(replicability_score)
                    result = pd.DataFrame(scores_per_dimension, columns=[options], index=idx)
                    total_result.append(result)
                table = pd.concat(total_result)
                self.sd = choosen_dimension
                return table
        else:
            return "Option must be inside " + choosen_dimension + " dimension."
        
    def replicability_comparison(self, option: str, mode = 0):# to check changes of one option if the other configurations are changed
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get("dimensions")
        q = data.get("query")
        
        if option not in d[self.sd]:
            raise Exception("choose option from selected dimension")    
        if len(d[self.sd]) != 2:
            raise Exception("choosen dimension option must be 2 to make comparison")     
        else:
            dimensions = []
            for key,value in d.items():
                if key != self.sd:
                    dimensions.append(key)
            
            if mode == 1:
                df = self.calculateRank(option)['Rank 1']
                df = df.sort_index(kind='stable')
                idx = list(df.index)
                df = df.to_list()
                
                total_score = []
                for x in df:
                    scores = x / q
                    total_score.append(scores)
                    
                table = pd.DataFrame(total_score, index=idx, columns=[option])
                table.index = table.index.str.replace(option, '')
                table.index = table.index.str.replace('.', ' ', regex=True)
                return table
            elif mode == 0:
                reversed_order = dimensions[::-1]
                total_scores = []
                for x in range(len(dimensions)):
                    scores_per_dimension = []
                    idx = []
                    for i in d[dimensions[x]]:
                        df = self.calculateRank(option, i)['Rank 1']
                        idx.append(i)
                        df = df.tolist()
                        replicability_score = sum(df)/(q*len(d[reversed_order[x]]))
                        scores_per_dimension.append(replicability_score)
                    result = pd.DataFrame(scores_per_dimension, columns=[dimensions[x]], index=idx)
                    total_scores.append(result)
                global_replicability_scores = pd.concat(total_scores)
                global_replicability_scores = global_replicability_scores.replace(np.nan, '')
                return global_replicability_scores

    def calculateRank(self, *args):
        global rank_dataframe_rscore

        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')

        if self.sd not in d:
            raise Exception("incorrect dimension")

        # splitting dataframe according to SD
        load = FileReader(self.config_path, self.log_path, self.size, self.sd)
        delRows = []
        df = load.file_reader()
        delete_config = load.deleteConfig()

        for arg in args:
            if isinstance(arg, list):
                for x in arg:
                    if not isinstance(x, int):
                        raise TypeError('list must be an integer')
                    else:
                        x -= 1
                        delRows.append(x)
                # df = load.file_reader()
                df = df.drop(df.columns[delRows], axis=1)
            else:
                pass
        Dictionary = {}
        count = 0
        loop = len(d.get(self.sd))
        dfs = []
        for i in range(int(len(df.index)/len(d.get(self.sd)))):
            Dictionary['df_{}'.format(i)] = df[count:loop]
            count = loop
            loop = loop+len(d.get(self.sd))
            dfs.append(Dictionary[f'df_{i}'])

        rank_dataframe = []
        # create rank occurences
        for x in dfs:
            df_ranks = x.T
            column_names = df_ranks.columns.to_numpy().tolist()

            df_ranks_occurences = []
            for index, row in df_ranks.iterrows():
                df_ranks_occurences.append(ss.rankdata(row, method='max'))

            df_ranks_occurences = pd.DataFrame(df_ranks_occurences)

            df_transpose = df_ranks_occurences.transpose()

            rank_table = []
            for index, row in df_transpose.iterrows():
                result_row = np.zeros(len(df_transpose.index))
                for i in range(len(row)):
                    result_row[int(row[i])-1] += 1
                rank_table.append(result_row)

            rank_table = pd.DataFrame(rank_table)
            rank_table = rank_table.set_axis(column_names, axis='index')
            rank_table = rank_table.set_axis(
                ['Rank '+str(i+1) for i in range(len(column_names))], axis='columns')
    
        # CREATE R SCORE
            q = len(df_transpose.columns)
            d = len(rank_table.index)

            rank_score = []
            for index, row in rank_table.iterrows():
                s = 0
                if d == 1:
                    s = 1
                else:
                    for r in range(d):
                        s = s + (row[r]*(d-(r+1)) / (q*(d-1)))
                rank_score.append(s)
            rank_score = pd.DataFrame(rank_score)
            rank_score = rank_score.set_axis(column_names, axis='index')
            rank_score = rank_score.set_axis(['Result'], axis='columns')
            rank_score = pd.concat([rank_table, rank_score], axis=1)
            rank_dataframe.append(rank_score)
        rank_dataframe_rscore = pd.concat(rank_dataframe)

        # if len(delete_config) == 0:
        #     pass
        # elif len(delete_config) != 0:
        #     rank_dataframe_rscore = rank_dataframe_rscore.drop(delete_config, axis=0)

        if len(args) == 0:
            return rank_dataframe_rscore.sort_values(by=['Result'], ascending=False)
        elif len(args) != 0:
            options = []
            store = []
            for arg in args:
                if isinstance(arg, str):
                    options.append(arg)
                else:
                    pass
            for x in range(len(options)):
                c = c = fr'(?=.*\b{options[x]}\b)'
                store.append(c)
            command = "".join(store)
            rank_dataframe_rscore = rank_dataframe_rscore.loc[rank_dataframe_rscore.index.str.contains(
                command, regex=True)]
            return rank_dataframe_rscore.sort_values(by=['Result'], ascending=False)

    def plot(self, view: str):
        self.calculateRank()
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        options = list(d.values())
        dims = list(d.keys())
        d = list(d.keys())

        if len(dims) > 3:
            raise Exception("can only plot 3 dimensions")

        if self.sd == dims[0]:
            if view not in options[0]:
                filter = rank_dataframe_rscore.loc[rank_dataframe_rscore.index.str.contains(
                    fr'\b{view}\b', regex=True)]

                if view in options[1]:
                    Dictionary = {}
                    loop = len(options[0])
                    count = 0
                    for i in options[2]:
                        Dictionary['{}'.format(i)] = filter[count:loop]
                        count = loop
                        loop = loop+len(options[0])

                    key = list(Dictionary)
                    Dictionary2 = {}

                    for j in range(len(options[2])):
                        val = list(Dictionary[key[j]]['Result'])
                        Dictionary2['val{}'.format(j)] = val

                    data = np.vstack(list(Dictionary2.values()))
                    df2 = pd.DataFrame(data=data, index=list(
                        Dictionary.keys()), columns=options[0])
                    return df2.plot.bar(title=str(self.sd).capitalize() + " SD Rank pivoting " + str(d[2]).capitalize() + " formats for " + str(view).capitalize() + " " + str(d[1]).capitalize(), rot=0, fontsize=14, figsize=(10, 10))

                elif view in options[2]:
                    Dictionary = {}
                    loop = len(options[0])
                    count = 0
                    for i in options[1]:
                        Dictionary['{}'.format(i)] = filter[count:loop]
                        count = loop
                        loop = loop+len(options[0])

                    key = list(Dictionary)
                    Dictionary2 = {}

                    for j in range(len(options[1])):
                        val = list(Dictionary[key[j]]['Result'])
                        Dictionary2['val{}'.format(j)] = val

                    data = np.vstack(list(Dictionary2.values()))
                    df2 = pd.DataFrame(data=data, index=list(
                        Dictionary.keys()), columns=options[0])
                    return df2.plot.bar(title=str(self.sd).capitalize() + " SD Rank pivoting " + str(d[1]).capitalize() + " formats for " + str(view).capitalize() + " " + str(d[2]).capitalize(), rot=0, fontsize=14, figsize=(10, 10))
            else:
                return "the dimension is " + dims[0]

        elif self.sd == dims[1]:
            if view not in options[1]:
                filter = rank_dataframe_rscore.loc[rank_dataframe_rscore.index.str.contains(
                    fr'\b{view}\b', regex=True)]

                if view in options[2]:
                    Dictionary = {}
                    loop = len(options[1])
                    count = 0
                    for i in options[0]:
                        Dictionary['{}'.format(i)] = filter[count:loop]
                        count = loop
                        loop = loop+len(options[1])

                    key = list(Dictionary)
                    Dictionary2 = {}

                    for j in range(len(options[0])):
                        val = list(Dictionary[key[j]]['Result'])
                        Dictionary2['val{}'.format(j)] = val

                    data = np.vstack(list(Dictionary2.values()))
                    df2 = pd.DataFrame(data=data, index=list(
                        Dictionary.keys()), columns=options[1])
                    return df2.plot.bar(title=str(self.sd).capitalize() + " SD Rank pivoting " + str(d[0]).capitalize() + " formats for " + str(view).capitalize() + " " + str(d[2]).capitalize(), rot=0, fontsize=14, figsize=(10, 10))

                elif view in options[0]:
                    Dictionary = {}
                    loop = len(options[1])
                    count = 0
                    for i in options[2]:
                        Dictionary['{}'.format(i)] = filter[count:loop]
                        count = loop
                        loop = loop+len(options[1])

                    key = list(Dictionary)
                    Dictionary2 = {}

                    for j in range(len(options[2])):
                        val = list(Dictionary[key[j]]['Result'])
                        Dictionary2['val{}'.format(j)] = val

                    data = np.vstack(list(Dictionary2.values()))
                    df2 = pd.DataFrame(data=data, index=list(
                        Dictionary.keys()), columns=options[1])
                    return df2.plot.bar(title=str(self.sd).capitalize() + " SD Rank pivoting " + str(d[2]).capitalize() + " formats for " + str(view).capitalize() + " " + str(d[0]).capitalize(), rot=0, fontsize=14, figsize=(10, 10))
            else:
                return "the dimension is " + dims[1]

        elif self.sd == dims[2]:
            if view not in options[2]:
                filter = rank_dataframe_rscore.loc[rank_dataframe_rscore.index.str.contains(
                    fr'\b{view}\b', regex=True)]

                if view in options[0]:
                    Dictionary = {}
                    loop = len(options[2])
                    count = 0
                    for i in options[1]:
                        Dictionary['{}'.format(i)] = filter[count:loop]
                        count = loop
                        loop = loop+len(options[2])

                    key = list(Dictionary)
                    Dictionary2 = {}

                    for j in range(len(options[1])):
                        val = list(Dictionary[key[j]]['Result'])
                        Dictionary2['val{}'.format(j)] = val

                    data = np.vstack(list(Dictionary2.values()))
                    df2 = pd.DataFrame(data=data, index=list(
                        Dictionary.keys()), columns=options[2])
                    return df2.plot.bar(title=str(self.sd).capitalize() + " SD Rank pivoting " + str(d[1]).capitalize() + " formats for " + str(view).capitalize() + " " + str(d[0]).capitalize(), rot=0, fontsize=14, figsize=(10, 10))

                elif view in options[1]:
                    Dictionary = {}
                    loop = len(options[2])
                    count = 0
                    for i in options[0]:
                        Dictionary['{}'.format(i)] = filter[count:loop]
                        count = loop
                        loop = loop+len(options[2])

                    key = list(Dictionary)
                    Dictionary2 = {}

                    for j in range(len(options[0])):
                        val = list(Dictionary[key[j]]['Result'])
                        Dictionary2['val{}'.format(j)] = val

                    data = np.vstack(list(Dictionary2.values()))
                    df2 = pd.DataFrame(data=data, index=list(
                        Dictionary.keys()), columns=options[2])
                    return df2.plot.bar(title=str(self.sd).capitalize() + " SD Rank pivoting " + str(d[0]).capitalize() + " formats for " + str(view).capitalize() + " " + str(d[1]).capitalize(), rot=0, fontsize=14, figsize=(10, 10))
            else:
                return "the dimension is " + dims[2]

    def plotRadar(self):
        warnings.simplefilter(action='ignore', category=FutureWarning)

        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        dims = list(d.keys())

        if len(dims) > 3:
            raise Exception("can only plot 3 dimensions")

        topConfig = self.calculateRank()['Result'].index.str.replace(
            '.', ' ', regex=False)
        topConfig = topConfig[0].split()

        x = topConfig[0]
        y = topConfig[1]
        z = topConfig[2]
        res = []

        for i in dims:
            self.sd = i
            self.calculateRank()
            r = rank_dataframe_rscore.loc[rank_dataframe_rscore.index.str.contains(
                fr'(?=.*\b{x}\b)(?=.*\b{y}\b)(?=.*\b{z}\b)', regex=True)]['Result']
            res.append(r[0])
        
        tet = []
        count = 0
        for j in dims:
            t = j + ": " + topConfig[count]
            tet.append(t)
            count += 1

        df = pd.DataFrame(dict(r=res, theta=tet))
        fig = px.line_polar(df, labels={"a": "pler"}, r='r',
                            theta='theta', line_close=True)
        return fig.show()

    def plotBox(self, q=None):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')

        if self.sd not in d:
            raise Exception("incorrect dimension")

        # splitting dataframe according to SD
        load = FileReader(self.config_path, self.log_path, self.size, self.sd)
        df = load.file_reader()
        df = df.head(10).reset_index()
        df.rename(columns={'index': 'conf'}, inplace=True)

        plt.figure(figsize=(15, 10))

        # box plot in horizontal mode
        if q != None:
            box_plot = sns.boxplot(data=df[q], palette='rainbow',
                                   orient='v',  width=.25, showfliers=True)
        elif q == None:
            box_plot = sns.boxplot(data=df, palette='rainbow',
                                   orient='v',  width=.25, showfliers=True)

        i = 0
        for xtick in box_plot.get_xticklabels():
            Q = xtick.get_text()
            min_val = boxplot_stats(df[Q])[0]['whislo']
            max_val = max(boxplot_stats(df[Q])[0]['fliers']) if (
                len(boxplot_stats(df[Q])[0]['fliers'] != 0)) else 0
            whisk_max = boxplot_stats(df[Q])[0]['whishi']

            minConf_text = df.loc[df[Q] == min_val, 'conf'].iloc[0]
            maxConf_text = df.loc[df[Q] == max_val,
                                  'conf'].iloc[0] if (max_val != 0) else ""
            whisk_max_text = df.loc[df[Q] == whisk_max, 'conf'].iloc[0]

            box_plot.text(i, min_val, minConf_text, horizontalalignment='center',
                          color='g', size='large', weight='bold')
            box_plot.text(i, max_val, maxConf_text, horizontalalignment='center',
                          color='r', size='large', weight='bold')
            box_plot.text(i, whisk_max, whisk_max_text, horizontalalignment='center',
                          color='r', size='large', weight='bold')
            i = i+1

        box_plot.set_xlabel("Query", fontsize=15)
        box_plot.set_ylabel("Runtime (in sec)", fontsize=15)
        box_plot.set(title='Query Box Plot')
        return plt.show()


# MULTI DIMENSION


class MDRank(FileReader):
    def __init__(self, config_path: str, log_path: str, size: str, sd=None):
        super().__init__(config_path, log_path, size, sd)

    def dominates(self, row, candidateRow):
        # kalo lebih kecil semua berarti dominasi
        return np.sum([row[x] <= candidateRow[x] for x in range(len(row))]) == len(row)

    def dominates_agg(self, row, candidateRow):
        return np.sum([row[x] >= candidateRow[x] for x in range(len(row))]) == len(row)

    def getConfs(self, points, orignaldf):
        point_confs = {}
        for i in range(0, len(orignaldf)):
            for val in points:
                if (list(val) == np.array(orignaldf[i][1:], dtype=np.float64).tolist()):
                    point_confs[val] = orignaldf[i][0]
        return point_confs

    def getConfsSorted_Q(self, confsDict):
        # # summing all the values using sum()
        temp1 = {val: sum(float(idx) for idx in val)
                 for val, key in confsDict.items()}  # sum the ranks of non dominated solution

        # using sorted to perform sorting as required
        # sort according to the sum before
        temp2 = sorted(
            temp1.items(), key=lambda ele: temp1[ele[0]], reverse=False)
        res = {}
        for key, val in temp2:
            res[key] = confsDict[key]
        return res

    def getConfsSorted_Agg(self, confsDict):
        # # summing all the values using sum()
        temp1 = {val: sum(float(idx) for idx in val)
                 for val, key in confsDict.items()}  # sum the ranks of non dominated solution

        # using sorted to perform sorting as required
        # sort according to the sum before
        temp2 = sorted(
            temp1.items(), key=lambda ele: temp1[ele[0]], reverse=True)
        res = {}
        for key, val in temp2:
            res[key] = confsDict[key]
        return res

    def paretoQ(self, q=None):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')

        self.sd = list(d.keys())[-1]

        if q is not None:
            if isinstance(q, list):
                dimensionsAll = getRanks(
                    self.config_path, self.log_path, self.size, self.sd).getRanks(q)
                dimensionsAll = dimensionsAll.reset_index().values
                dimensions = np.array(getRanks(
                    self.config_path, self.log_path, self.size, self.sd).getRanks(q)[:], dtype=np.float64)
            else:
                raise Exception("parameter must be a list")
        elif q is None:
            dimensionsAll = getRanks(
                self.config_path, self.log_path, self.size, self.sd).getRanks()
            dimensionsAll = dimensionsAll.reset_index().values
            dimensions = np.array(getRanks(
                self.config_path, self.log_path, self.size, self.sd).getRanks()[:], dtype=np.float64)

        inputPoints = dimensions.tolist()
        paretoPoints, dominatedPoints = Nsga2(
            inputPoints, self.dominates).execute()

        pareto_q = self.getConfs(paretoPoints, dimensionsAll)
        pareto_q = self.getConfsSorted_Q(pareto_q)

        vals = np.array(list(pareto_q.values()))
        table_pareto = pd.DataFrame({'Solution': vals}, columns=['Solution'])

        dominated = self.getConfs(dominatedPoints, dimensionsAll)
        dominated = self.getConfsSorted_Q(dominated)
        vals = np.array(list(dominated.values()))
        table_dominated = pd.DataFrame(
            {'Dominated': vals}, columns=['Dominated'])

        table = pd.concat([table_pareto, table_dominated], axis=1)
        table = table.replace(np.nan, '', regex=True)
        return table

    def paretoAgg(self, q=None):
        global dominatedPoints
        global paretoPoints
        global dims

        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        dims = list(d.keys())

        li = []
        if q is not None:
            if isinstance(q, list):
                for x in dims:
                    self.sd = x
                    dimension_ranking = SDRank(
                        self.config_path, self.log_path, self.size, self.sd).calculateRank(q)
                    dimension_ranking = dimension_ranking['Result']
                    li.append(dimension_ranking)
            else:
                raise Exception("parameter must be a list")
        elif q is None:
            for x in dims:
                self.sd = x
                dimension_ranking = SDRank(
                    self.config_path, self.log_path, self.size, self.sd).calculateRank()
                dimension_ranking = dimension_ranking['Result']
                li.append(dimension_ranking)

        new_df = pd.concat(li, axis=1)
        dimensionsAll = new_df.reset_index().values
        dimensions = np.array(dimensionsAll[:, 1:], dtype=np.float64)
        inputPoints = dimensions.tolist()
        paretoPoints, dominatedPoints = Nsga2(
            inputPoints, self.dominates_agg).execute()
        pareto_agg = self.getConfs(paretoPoints, dimensionsAll)
        pareto_agg = self.getConfsSorted_Agg(pareto_agg)
        vals = np.array(list(pareto_agg.values()))
        table_pareto = pd.DataFrame(vals, columns=['Solution'])

        dominated = self.getConfs(dominatedPoints, dimensionsAll)
        dominated = self.getConfsSorted_Agg(dominated)
        vals = np.array(list(dominated.values()))
        table_dominated = pd.DataFrame(
            {'Dominated': vals}, columns=['Dominated'])

        table = pd.concat([table_pareto, table_dominated], axis=1)
        table = table.replace(np.nan, '', regex=True)
        return table

    def plot(self):
        self.paretoAgg()

        if len(dims) > 3:
            raise Exception("can only plot 3 dimensions")

        fig = plt.figure()
        fig.set_size_inches(15, 15)
        ax = fig.add_subplot(111, projection='3d')
        dp = np.array(list(dominatedPoints))
        pp = np.array(list(paretoPoints))

        print(pp.shape, dp.shape)
        ax.scatter(dp[:, 0], dp[:, 1], dp[:, 2])
        ax.scatter(pp[:, 0], pp[:, 1], pp[:, 2], color='green')

        ax.set_xlabel('Rf')
        ax.set_ylabel('Rp')
        ax.set_zlabel('Rs')
        
        # if pp[:, 1].all == 1:
        if np.all(pp[:, 1] == pp[0, 1]) or np.all(pp[:, 0] == pp[0, 0]) or np.all(pp[:, 2] == pp[0, 2]) :
            return plt.show()
        else:
            import matplotlib.tri as mtri
            triang = mtri.Triangulation(pp[:, 0], pp[:, 1])
            ax.plot_trisurf(triang, pp[:, 2], color='green', alpha=0.3)
            return plt.show()

class RTA(FileReader):
    def __init__(self, config_path: str, log_path: str, size: str, sd=None):
        super().__init__(config_path, log_path, size, sd)

    def rta(self):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        dims = list(d.keys())
        
        if len(dims) <= 3:
            dicts = {}
            for x in dims:
                self.sd = x
                r_scores = SDRank(self.config_path, self.log_path, self.size, self.sd).calculateRank()
                r_scores = r_scores.sort_index(kind = "stable")
                idx = list(r_scores.index)
                # print(idx)
                # print(r_scores)
                # score_list.append(r_scores)
                dicts[x] = r_scores['Result'].to_list()
            # print(dicts)
            # print(idx)
            values = list(dicts.values())
            
            rta_scores = []
            for i in range (len(idx)):
                rs = values[0][i]
                rp = values[1][i]
                rf = values[2][i]
                # RTA Formula
                sin = math.sin(math.radians(120))
                y = sin/2
                total = y * (1+1+1)
                result = y * (rf*rp + rs*rp + rf*rs)
                triangle_area = total - result
                rta_scores.append(triangle_area)

            data = pd.DataFrame(rta_scores, index = idx, columns=["Rta_Scores"])
            data = data.sort_values('Rta_Scores', ascending=True, kind='stable')
            return data
        
        elif len(dims) > 3:
            raise Exception("RTA cannot calculate more than 3 dimensions")

            


class Nsga2:
    def __init__(self, inputPoints, dominates):
        self.inputPoints = inputPoints
        self.dominates = dominates

    def execute(self):
        paretoPoints = set()
        candidateRowNr = 0
        dominatedPoints = set()
        while True:
            # masuk row pertama
            candidateRow = self.inputPoints[candidateRowNr]
            self.inputPoints.remove(candidateRow)  # buang row pertamanya
            rowNr = 0
            nonDominated = True  # row kandidat yg baru masuk di set true buat non dominated nya
            while len(self.inputPoints) != 0 and rowNr < len(self.inputPoints):
                row = self.inputPoints[rowNr]  # ambil row lanjutannya
                # ngecek row kandidat ama row lanjutan di input_point dominasinya
                if self.dominates(candidateRow, row):
                    # If it is worse on all features remove the row from the array
                    self.inputPoints.remove(row)
                    dominatedPoints.add(tuple(row))
                elif self.dominates(row, candidateRow):
                    nonDominated = False
                    dominatedPoints.add(tuple(candidateRow))
                    rowNr += 1
                else:
                    rowNr += 1

            if nonDominated:
                # add the non-dominated point to the Pareto frontier
                paretoPoints.add(tuple(candidateRow))

            if len(self.inputPoints) == 0:
                break
        return paretoPoints, dominatedPoints



from file_reader import FileReader
import pandas as pd
import scipy.stats as ss


class getRanks(FileReader):
    def __init__(self, config_path: str, log_path: str, size: str, sd=None):
        super().__init__(config_path, log_path, size, sd)

    def getRanks(self, q=None):
        load = FileReader(self.config_path, self.log_path, self.size, self.sd)
        df = load.file_reader()
        delRows = []

        if q is not None:
            for x in q:
                if not isinstance(x, int):
                    raise TypeError('list must be an integer')
                else:
                    x -= 1
                    delRows.append(x)
            df = df.drop(df.columns[delRows], axis=1)
        elif q is None:
            pass

        df_transpose = df.T
        df_full_ranks = []
        for index, row in df_transpose.iterrows():
            df_full_ranks.append(ss.rankdata(row, method='max'))
        df_full_ranks = pd.DataFrame(df_full_ranks)
        df_full_ranks = df_full_ranks.T
        df_full_ranks = df_full_ranks.set_axis(
            df_transpose.columns, axis='index')  # configurations
        df_full_ranks = df_full_ranks.set_axis(
            [i+1 for i in range(len(df_transpose.index))], axis='columns')  # query

        return df_full_ranks



from Rank import SDRank, MDRank, RTA


class bestOfSD(SDRank):
    def __init__(self, config_path: str, log_path: str, size: str, sd: str, k: int):
        super().__init__(config_path, log_path, size, sd)
        self.k = k

    def bestOfSD(self):
        df = SDRank(self.config_path, self.log_path,
                    self.size, self.sd).calculateRank()
        df = df['Result']
        df = df.nlargest(self.k)
        return df.index.to_numpy().tolist()


class bestOfParetoQ(MDRank):
    def __init__(self, config_path: str, log_path: str, size: str, k: int, sd=None):
        super().__init__(config_path, log_path, size, sd)
        self.k = k

    def bestOfParetoQ(self):
        df = MDRank(self.config_path, self.log_path,
                    self.size, self.sd).paretoQ()
        df = list(df['Solution'])[:self.k]
        return df


class bestOfParetoAgg(MDRank):
    def __init__(self, config_path: str, log_path: str, size: str, k: int, sd=None):
        super().__init__(config_path, log_path, size, sd)
        self.k = k

    def bestOfParetoAgg(self):
        df = MDRank(self.config_path, self.log_path,
                    self.size, self.sd).paretoAgg()
        df = list(df['Solution'])[:self.k]
        return df


class bestOfRTA(RTA):
    def __init__(self, config_path: str, log_path: str, size: str, k: int, sd=None):
        super().__init__(config_path, log_path, size, sd)
        self.k = k

    def bestOfRTA(self):
        df = RTA(self.config_path, self.log_path, self.size, self.sd).rta()
        df = list(df.index)
        return df[:self.k]



import yaml
class Loader:
    def __init__(self, config_path:str):
        self.config_path = config_path
    
    def loader(self):
        with open(self.config_path, "r") as f:
            data = yaml.safe_load(f)
        return data


from logging import exception
from tkinter.ttk import Style
from turtle import color, width
from PAPyA.config_loader import Loader
from PAPyA.get_ranks import getRanks
from PAPyA.best_of_k import bestOfParetoAgg, bestOfParetoQ, bestOfRTA, bestOfSD
from PAPyA.file_reader import FileReader
from PAPyA.kendall_index import kendallIndex
from Rank import SDRank
from Rank import MDRank
from Rank import RTA
import pandas as pd
import seaborn as sns
from matplotlib import pyplot as plt
import numpy as np

# RANKING VALIDATION


class Conformance(FileReader):
    def __init__(self, config_path: str, log_path: str, size: str, li: list, k: int, h: int, sd=None):
        super().__init__(config_path, log_path, size, sd)
        self.k = k
        self.h = h
        self.li = li

    def run(self):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        query = data.get('query')
        dims = list(d.keys())

        data = []
        idx = []

        for x in self.li:
            if x in dims:
                self.sd = x
                criteria_table = getRanks(
                    self.config_path, self.log_path, self.size, self.sd).getRanks()
                criteria_table = criteria_table.loc[bestOfSD(
                    self.config_path, self.log_path, self.size, self.sd, self.k).bestOfSD()]
                criteria_table = criteria_table[criteria_table > self.h]
                count = criteria_table.count(axis=1)
                sum = count.sum(axis=0)
                conformance = 1-(sum/(self.k*query))
                data.append(conformance)
                idx.append(x)
            elif x == 'paretoQ':
                self.sd = list(d.keys())[-1]
                criteria_table_paretoQ = getRanks(
                    self.config_path, self.log_path, self.size, self.sd).getRanks()
                criteria_table_paretoQ = criteria_table_paretoQ.loc[bestOfParetoQ(
                    self.config_path, self.log_path, self.size, self.k).bestOfParetoQ()]
                criteria_table_paretoQ = criteria_table_paretoQ[criteria_table_paretoQ > self.h]
                count = criteria_table_paretoQ.count(axis=1)
                sum = count.sum(axis=0)
                conformance = 1 - (sum/(self.k*query))
                data.append(conformance)
                idx.append('paretoQ')
            elif x == 'paretoAgg':
                self.sd = list(d.keys())[-1]
                criteria_table_paretoAgg = getRanks(
                    self.config_path, self.log_path, self.size, self.sd).getRanks()
                criteria_table_paretoAgg = criteria_table_paretoAgg.loc[bestOfParetoAgg(
                    self.config_path, self.log_path, self.size, self.k).bestOfParetoAgg()]
                criteria_table_paretoAgg = criteria_table_paretoAgg[criteria_table_paretoAgg > self.h]
                count = criteria_table_paretoAgg.count(axis=1)
                sum = count.sum(axis=0)
                conformance = 1 - (sum/(self.k*query))
                data.append(conformance)
                idx.append('paretoAgg')
            elif x == 'RTA':
                self.sd = list(d.keys())[-1]
                criteria_table_RTA = getRanks(
                    self.config_path, self.log_path, self.size, self.sd).getRanks()
                criteria_table_RTA = criteria_table_RTA.loc[bestOfRTA(
                    self.config_path, self.log_path, self.size, self.k).bestOfRTA()]
                criteria_table_RTA = criteria_table_RTA[criteria_table_RTA > self.h]
                count = criteria_table_RTA.count(axis=1)
                sum = count.sum(axis=0)
                conformance = 1 - (sum/(self.k*query))
                data.append(conformance)
                idx.append('RTA')

        table = pd.DataFrame(data, index=idx, columns=[self.size])
        return table

    def plot(self, mode=0):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        query = data.get('query')
        dims = list(d.keys())

        if mode == 0:
            data = self.run()
            value = data[self.size].to_list()
            idx = list(data.index)

            non_conformance = []
            for x in value:
                stacked = 1 - x
                non_conformance.append(stacked)

            df = pd.DataFrame(np.transpose(np.array([value, non_conformance])), index=idx, columns=[
                              "Conformance", "Non-Conformance"])
            sns.set(style='white', rc = {'figure.figsize':(10,6)})
            cmap = sns.color_palette()
            df.plot(kind='bar', stacked=True, color=[cmap[2], cmap[3]], width=0.2)
            plt.xticks(rotation='0')

            return plt.show()

        elif mode == 1:
            total_conformance = []
            config_name = []
            for x in self.li:
                if x in dims:
                    self.sd = x
                    criteria_table = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table = criteria_table.loc[bestOfSD(
                        self.config_path, self.log_path, self.size, self.sd, self.k).bestOfSD()]
                    criteria_table = criteria_table[criteria_table > self.h]
                    count = criteria_table.count(axis=1)
                    if len(count) >= 3:
                        count = count[:3]
                    elif len(count) < 3:
                        count = count[:]
                    idx = list(count.index)
                    val = count.to_list()

                    conformance_value = []
                    counter_value = []
                    for i in val:
                        conformance = 1-(i/query)
                        counter = 1 - conformance
                        conformance_value.append(conformance)
                        counter_value.append(counter)
                    total_conformance.append(conformance_value)
                    total_conformance.append(counter_value)
                    config_name.append(idx)
                elif x == 'paretoQ':
                    self.sd = list(d.keys())[-1]
                    criteria_table_paretoQ = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table_paretoQ = criteria_table_paretoQ.loc[bestOfParetoQ(
                        self.config_path, self.log_path, self.size, self.k).bestOfParetoQ()]
                    criteria_table_paretoQ = criteria_table_paretoQ[criteria_table_paretoQ > self.h]
                    count = criteria_table_paretoQ.count(axis=1)
                    if len(count) >= 3:
                        count = count[:3]
                    elif len(count) < 3:
                        count = count[:]
                    idx = list(count.index)
                    val = count.to_list()

                    conformance_value = []
                    counter_value = []
                    for i in val:
                        conformance = 1-(i/query)
                        counter = 1 - conformance
                        conformance_value.append(conformance)
                        counter_value.append(counter)
                    total_conformance.append(conformance_value)
                    total_conformance.append(counter_value)
                    config_name.append(idx)
                elif x == "paretoAgg":
                    self.sd = list(d.keys())[-1]
                    criteria_table_paretoAgg = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table_paretoAgg = criteria_table_paretoAgg.loc[bestOfParetoAgg(
                        self.config_path, self.log_path, self.size, self.k).bestOfParetoAgg()]
                    criteria_table_paretoAgg = criteria_table_paretoAgg[
                        criteria_table_paretoAgg > self.h]
                    count = criteria_table_paretoAgg.count(axis=1)
                    if len(count) >= 3:
                        count = count[:3]
                    elif len(count) < 3:
                        count = count[:]
                    idx = list(count.index)
                    val = count.to_list()

                    conformance_value = []
                    counter_value = []
                    for i in val:
                        conformance = 1-(i/query)
                        counter = 1 - conformance
                        conformance_value.append(conformance)
                        counter_value.append(counter)
                    total_conformance.append(conformance_value)
                    total_conformance.append(counter_value)
                    config_name.append(idx)
                elif x == "RTA":
                    self.sd = list(d.keys())[-1]
                    criteria_table_RTA = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table_RTA = criteria_table_RTA.loc[bestOfRTA(
                        self.config_path, self.log_path, self.size, self.k).bestOfRTA()]
                    criteria_table_RTA = criteria_table_RTA[criteria_table_RTA > self.h]
                    count = criteria_table_RTA.count(axis=1)
                    if len(count) >= 3:
                        count = count[:3]
                    elif len(count) < 3:
                        count = count[:]
                    idx = list(count.index)
                    val = count.to_list()

                    conformance_value = []
                    counter_value = []
                    for i in val:
                        conformance = 1-(i/query)
                        counter = 1 - conformance
                        conformance_value.append(conformance)
                        counter_value.append(counter)
                    total_conformance.append(conformance_value)
                    total_conformance.append(counter_value)
                    config_name.append(idx)

            counter = 0
            for i in range(len(self.li)):
                bottom = total_conformance[counter]
                top = total_conformance[counter+1]
                print(bottom, top)
                counter = counter + 2

                xlabel = config_name[i]
                cmap = sns.color_palette()

                plt.title(f"{self.li[i]} ranking")
                plt.bar(xlabel, bottom, color=[cmap[2]], width=0.2)
                plt.bar(xlabel, top, bottom=bottom, color=[cmap[3]], width=0.2)
                plt.show()

    def configurationQueryRanks(self, dimension: str, mode=0):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        if mode == 0:
            if dimension in self.li:
                if dimension == "paretoQ":
                    self.sd = list(d.keys())[-1]
                    criteria_table_paretoQ = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table_paretoQ = criteria_table_paretoQ.loc[bestOfParetoQ(
                        self.config_path, self.log_path, self.size, self.k).bestOfParetoQ()]
                    return criteria_table_paretoQ
                elif dimension == "paretoAgg":
                    self.sd = list(d.keys())[-1]
                    criteria_table_paretoAgg = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table_paretoAgg = criteria_table_paretoAgg.loc[bestOfParetoAgg(
                        self.config_path, self.log_path, self.size, self.k).bestOfParetoAgg()]
                    return criteria_table_paretoAgg
                elif dimension == "RTA":
                    self.sd = list(d.keys())[-1]
                    criteria_table_RTA = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table_RTA = criteria_table_RTA.loc[bestOfRTA(
                        self.config_path, self.log_path, self.size, self.k).bestOfRTA()]
                    return criteria_table_RTA
                else:
                    self.sd = dimension
                    criteria_table = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table = criteria_table.loc[bestOfSD(
                        self.config_path, self.log_path, self.size, self.sd, self.k).bestOfSD()]
                    return criteria_table

            elif dimension not in self.li:
                raise Exception("Dimension is not in set")

        elif mode == 1:
            def color_boolean(val):
                color = ''
                if val == 'True':
                    color = 'red'
                elif val == 'False':
                    color = 'green'
                return 'color: %s' % color

            if dimension in self.li:
                if dimension == "paretoQ":
                    self.sd = list(d.keys())[-1]
                    criteria_table_paretoQ = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table_paretoQ = criteria_table_paretoQ.loc[bestOfParetoQ(
                        self.config_path, self.log_path, self.size, self.k).bestOfParetoQ()]
                    criteria_table_paretoQ = criteria_table_paretoQ[criteria_table_paretoQ > self.h]
                    ordering = criteria_table_paretoQ.count(axis='columns')
                    ordering = ordering.sort_values(ascending=True)
                    ordering = ordering.index.to_list()
                    criteria_table_paretoQ = criteria_table_paretoQ.loc[ordering]
                    criteria_table_paretoQ = criteria_table_paretoQ[:].notnull(
                    )
                    criteria_table_paretoQ = criteria_table_paretoQ[:].astype(
                        str)
                    criteria_table_paretoQ = criteria_table_paretoQ.style.applymap(
                        color_boolean)
                    return criteria_table_paretoQ
                elif dimension == "paretoAgg":
                    self.sd = list(d.keys())[-1]
                    criteria_table_paretoAgg = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table_paretoAgg = criteria_table_paretoAgg.loc[bestOfParetoAgg(
                        self.config_path, self.log_path, self.size, self.k).bestOfParetoAgg()]
                    criteria_table_paretoAgg = criteria_table_paretoAgg[
                        criteria_table_paretoAgg > self.h]
                    ordering = criteria_table_paretoAgg.count(axis='columns')
                    ordering = ordering.sort_values(ascending=True)
                    ordering = ordering.index.to_list()
                    criteria_table_paretoAgg = criteria_table_paretoAgg.loc[ordering]
                    criteria_table_paretoAgg = criteria_table_paretoAgg[:].notnull(
                    )
                    criteria_table_paretoAgg = criteria_table_paretoAgg[:].astype(
                        str)
                    criteria_table_paretoAgg = criteria_table_paretoAgg.style.applymap(
                        color_boolean)
                    return criteria_table_paretoAgg
                elif dimension == "RTA":
                    self.sd = list(d.keys())[-1]
                    criteria_table_RTA = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table_RTA = criteria_table_RTA.loc[bestOfRTA(
                        self.config_path, self.log_path, self.size, self.k).bestOfRTA()]
                    criteria_table_RTA = criteria_table_RTA[
                        criteria_table_RTA > self.h]
                    ordering = criteria_table_RTA.count(axis='columns')
                    ordering = ordering.sort_values(ascending=True)
                    ordering = ordering.index.to_list()
                    criteria_table_RTA = criteria_table_RTA.loc[ordering]
                    criteria_table_RTA = criteria_table_RTA[:].notnull(
                    )
                    criteria_table_RTA = criteria_table_RTA[:].astype(
                        str)
                    criteria_table_RTA = criteria_table_RTA.style.applymap(
                        color_boolean)
                    return criteria_table_RTA
                else:
                    self.sd = dimension
                    criteria_table = getRanks(
                        self.config_path, self.log_path, self.size, self.sd).getRanks()
                    criteria_table = criteria_table.loc[bestOfSD(
                        self.config_path, self.log_path, self.size, self.sd, self.k).bestOfSD()]
                    criteria_table = criteria_table[criteria_table > self.h]
                    ordering = criteria_table.count(axis='columns')
                    ordering = ordering.sort_values(ascending=True)
                    ordering = ordering.index.to_list()
                    criteria_table = criteria_table.loc[ordering]
                    criteria_table = criteria_table[:].notnull()
                    criteria_table = criteria_table[:].astype(str)
                    criteria_table = criteria_table.style.applymap(
                        color_boolean)
                    return criteria_table

            elif dimension not in self.li:
                raise Exception("Dimension is not in set")


class Coherence(FileReader):
    def __init__(self, config_path: str, log_path: str, li: list, sd=None, size=None):
        super().__init__(config_path, log_path, size, sd)
        self.li = li

    def run(self, rankset1: str, rankset2: str):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        dims = list(d.keys())

        result = []
        for x in self.li:
            if x in dims:
                var1 = SDRank(self.config_path, self.log_path,
                              rankset1, x).calculateRank()
                var1 = list(var1.index)
                var2 = SDRank(self.config_path, self.log_path,
                              rankset2, x).calculateRank()
                var2 = list(var2.index)
                kendall = kendallIndex(
                    var1, var2).normalised_kendall_tau_distance()
                result.append(kendall)
            elif x == 'paretoAgg':
                self.sd = list(d.keys())[-1]
                # var1 = SDRank(self.config_path, self.log_path,
                #               rankset1, self.sd).calculateRank()
                paretoAggSolution = MDRank(
                    self.config_path, self.log_path, rankset1, self.sd).paretoAgg()
                
                solution_data = paretoAggSolution['Solution']
                solution_data.replace('', np.nan, inplace=True)
                solution_data.dropna(inplace=True)
                dominated_data = paretoAggSolution['Dominated']
                dominated_data.replace('', np.nan, inplace=True)
                dominated_data.dropna(inplace=True)
                data1 = pd.concat([solution_data, dominated_data],
                                  axis=0, ignore_index=True)
                # paretoAggSolution = paretoAggSolution['Solution'].replace(
                #     '', np.nan)
                # paretoAggSolution = paretoAggSolution.dropna()
                # var1 = var1.loc[paretoAggSolution]
                # var1 = var1['Result']
                # var2 = SDRank(self.config_path, self.log_path,
                #               rankset2, self.sd).calculateRank()
                paretoAggSolution = MDRank(
                    self.config_path, self.log_path, rankset2, self.sd).paretoAgg()
                
                solution_data = paretoAggSolution['Solution']
                solution_data.replace('', np.nan, inplace=True)
                solution_data.dropna(inplace=True)
                dominated_data = paretoAggSolution['Dominated']
                dominated_data.replace('', np.nan, inplace=True)
                dominated_data.dropna(inplace=True)
                data2 = pd.concat([solution_data, dominated_data],
                                  axis=0, ignore_index=True)
                
                kendall = kendallIndex(
                    data1.tolist(), data2.tolist()).normalised_kendall_tau_distance()
                result.append(kendall)
                # paretoAggSolution = paretoAggSolution['Solution'].replace(
                #     '', np.nan)
                # paretoAggSolution = paretoAggSolution.dropna()
                # var2 = var2.loc[paretoAggSolution]
                # var2 = var2['Result']

                # newDf = pd.concat([var1, var2], axis=1)
                # newDf_column_name = newDf.columns.values
                # newDf_column_name[0] = str(rankset1)  # var1
                # newDf_column_name[1] = str(rankset2)  # var2

                # newDf.columns = newDf_column_name
                # newDf = newDf.fillna(1)
                # var1 = newDf[str(rankset1)]
                # var2 = newDf[str(rankset2)]

                # var1 = var1.to_numpy()
                # var2 = var2.to_numpy()
                
                # result.append(kendall)
            elif x == 'paretoQ':
                self.sd = list(d.keys())[-1]
                # var1 = SDRank(self.config_path, self.log_path,
                #               rankset1, self.sd).calculateRank()
                paretoQSolution = MDRank(
                    self.config_path, self.log_path, rankset1, self.sd).paretoQ()
                
                solution_data = paretoQSolution['Solution']
                solution_data.replace('', np.nan, inplace=True)
                solution_data.dropna(inplace=True)
                dominated_data = paretoQSolution['Dominated']
                dominated_data.replace('', np.nan, inplace=True)
                dominated_data.dropna(inplace=True)
                data1 = pd.concat([solution_data, dominated_data],
                                  axis=0, ignore_index=True)
                # paretoQSolution = paretoQSolution['Solution'].replace(
                #     '', np.nan)
                # paretoQSolution = paretoQSolution.dropna()
                # var1 = var1.loc[paretoQSolution]
                # var1 = var1['Result']
                # var2 = SDRank(self.config_path, self.log_path,
                #               rankset2, self.sd).calculateRank()
                paretoQSolution = MDRank(
                    self.config_path, self.log_path, rankset2, self.sd).paretoQ()
                
                solution_data = paretoQSolution['Solution']
                solution_data.replace('', np.nan, inplace=True)
                solution_data.dropna(inplace=True)
                dominated_data = paretoQSolution['Dominated']
                dominated_data.replace('', np.nan, inplace=True)
                dominated_data.dropna(inplace=True)
                data2 = pd.concat([solution_data, dominated_data],
                                  axis=0, ignore_index=True)
                
                kendall = kendallIndex(
                    data1.tolist(), data2.tolist()).normalised_kendall_tau_distance()
                result.append(kendall)
                # paretoQSolution = paretoQSolution['Solution'].replace(
                #     '', np.nan)
                # paretoQSolution = paretoQSolution.dropna()
                # var2 = var2.loc[paretoQSolution]
                # var2 = var2['Result']

                # newDf = pd.concat([var1, var2], axis=1)
                # newDf_column_name = newDf.columns.values
                # newDf_column_name[0] = str(rankset1)  # var1
                # newDf_column_name[1] = str(rankset2)  # var2

                # newDf.columns = newDf_column_name
                # newDf = newDf.fillna(1)
                # var1 = newDf[str(rankset1)]
                # var2 = newDf[str(rankset2)]

                # var1 = var1.to_numpy()
                # var2 = var2.to_numpy()
                
            elif x == "RTA":
                self.sd = list(d.keys())[-1]
                rta_result1 = RTA(self.config_path, self.log_path, rankset1).rta()
                data1 = list(rta_result1.index)
                rta_result2 = RTA(self.config_path, self.log_path, rankset2).rta()
                data2 = list(rta_result2.index)

                kendall = kendallIndex(
                    data1, data2).normalised_kendall_tau_distance()
                result.append(kendall)

        return pd.DataFrame(result, index=self.li, columns=['Kendall\'s Index'])

    def heatMapSubtract(self, *args, dimension: str):
        if dimension in self.li:
            if dimension == "paretoAgg":
                var = MDRank(self.config_path, self.log_path,
                             args[0]).paretoAgg()
                data1_1 = var['Solution']
                data1_1.replace('', np.nan, inplace=True)
                data1_1.dropna(inplace=True)
                data1_2 = var['Dominated']
                data1_2.replace('', np.nan, inplace=True)
                data1_2.dropna(inplace=True)
                data1 = pd.concat([data1_1, data1_2],
                                  axis=0, ignore_index=True)

                data1 = data1[:10]
                order1 = list(data1.index)
                idx = data1.tolist()
                order1 = [x+1 for x in order1]

                ylabels = []
                ordering = []
                for x in args[1:]:
                    var2 = MDRank(self.config_path,
                                  self.log_path, x).paretoAgg()
                    data2_1 = var2['Solution']
                    data2_1.replace('', np.nan, inplace=True)
                    data2_1.dropna(inplace=True)
                    data2_2 = var2['Dominated']
                    data2_2.replace('', np.nan, inplace=True)
                    data2_2.dropna(inplace=True)
                    data2 = pd.concat([data2_1, data2_2],
                                      axis=0, ignore_index=True)

                    data2 = pd.Series(data2.index.values, index=data2)
                    data2 = data2.loc[idx]
                    data2 = data2.tolist()
                    order2 = [x+1 for x in data2]

                    # CREATE YLABEL AND ORDERING
                    ylabels.append(str(args[0]) + "-" + str(x))
                    order1_2 = np.subtract(order1, order2)
                    order1_2 = np.absolute(order1_2)
                    ordering.append(order1_2)

                orders = np.stack((ordering), axis=0)
                xlabels = idx

                plt.rc('xtick', labelsize=8)
                plt.rc('ytick', labelsize=14)
                plt.figure(figsize=(22, 5))
                sns.heatmap(orders,
                            cmap='YlOrBr',
                            vmin=0,
                            xticklabels=xlabels,
                            yticklabels=ylabels,
                            annot=True,
                            square=True,
                            annot_kws={'fontsize': 8, 'fontweight': 'bold'})
                plt.yticks(rotation=0)
                plt.tick_params(
                    which='both',
                    bottom=False,
                    left=False,
                    labelbottom=False,
                    labeltop=True)
                return plt.tight_layout()

            elif dimension == "paretoQ":
                var = MDRank(self.config_path, self.log_path,
                             args[0]).paretoQ()
                data1_1 = var['Solution']
                data1_1.replace('', np.nan, inplace=True)
                data1_1.dropna(inplace=True)
                data1_2 = var['Dominated']
                data1_2.replace('', np.nan, inplace=True)
                data1_2.dropna(inplace=True)
                data1 = pd.concat([data1_1, data1_2],
                                  axis=0, ignore_index=True)

                data1 = data1[:10]
                order1 = list(data1.index)
                idx = data1.tolist()
                order1 = [x+1 for x in order1]

                ylabels = []
                ordering = []
                for x in args[1:]:
                    var2 = MDRank(self.config_path, self.log_path, x).paretoQ()
                    data2_1 = var2['Solution']
                    data2_1.replace('', np.nan, inplace=True)
                    data2_1.dropna(inplace=True)
                    data2_2 = var2['Dominated']
                    data2_2.replace('', np.nan, inplace=True)
                    data2_2.dropna(inplace=True)
                    data2 = pd.concat([data2_1, data2_2],
                                      axis=0, ignore_index=True)

                    data2 = pd.Series(data2.index.values, index=data2)
                    data2 = data2.loc[idx]
                    data2 = data2.tolist()
                    order2 = [x+1 for x in data2]

                    # CREATE YLABEL AND ORDERING
                    ylabels.append(str(args[0]) + "-" + str(x))
                    order1_2 = np.subtract(order1, order2)
                    order1_2 = np.absolute(order1_2)
                    ordering.append(order1_2)

                orders = np.stack((ordering), axis=0)
                xlabels = idx

                plt.rc('xtick', labelsize=8)
                plt.rc('ytick', labelsize=14)
                plt.figure(figsize=(22, 5))
                sns.heatmap(orders,
                            cmap='YlOrBr',
                            vmin=0,
                            xticklabels=xlabels,
                            yticklabels=ylabels,
                            annot=True,
                            square=True,
                            annot_kws={'fontsize': 8, 'fontweight': 'bold'})
                plt.yticks(rotation=0)
                plt.tick_params(
                    which='both',
                    bottom=False,
                    left=False,
                    labelbottom=False,
                    labeltop=True)
                return plt.tight_layout()

            elif dimension == "RTA":
                var = RTA(self.config_path, self.log_path, args[0]).rta()
                var = var['Rta_Scores']
                var = var.nsmallest(n=10, keep='first')
                idx = var.index.tolist()
                scores = var.to_numpy()
                order = np.argsort(scores, kind='stable')
                order = order + 1

                ylabels = []
                ordering = []
                for x in args[1:]:
                    var2 = RTA(self.config_path, self.log_path, x).rta()
                    scores = var2['Rta_Scores'].to_numpy()
                    val = np.argsort(scores, kind='stable')
                    # Put whatever series you want in its place
                    var2['Result'] = val
                    var2 = var2['Result']

                    var2 = var2.loc[idx]
                    order2_list = var2[:].tolist()
                    order2 = [x+1 for x in order2_list]
                    order2 = np.asarray(order2)

                    # CREATE YLABEL AND ORDERING
                    ylabels.append(str(args[0]) + "-" + str(x))
                    order1_2 = np.subtract(order, order2)
                    order1_2 = np.absolute(order1_2)
                    ordering.append(order1_2)
                
                orders = np.stack((ordering), axis=0)
                xlabels = idx
                
                plt.rc('xtick', labelsize=8)
                plt.rc('ytick', labelsize=14)
                plt.figure(figsize=(22, 5))
                sns.heatmap(orders,
                            cmap='YlOrBr',
                            vmin=0,
                            xticklabels=xlabels,
                            yticklabels=ylabels,
                            annot=True,
                            square=True,
                            annot_kws={'fontsize': 8, 'fontweight': 'bold'})
                plt.yticks(rotation=0)
                plt.tick_params(
                    which='both',
                    bottom=False,
                    left=False,
                    labelbottom=False,
                    labeltop=True)
                return plt.tight_layout()

            else:
                var = SDRank(self.config_path, self.log_path,
                             args[0], dimension).calculateRank()
                var = var['Result']
                var = var.nlargest(n=10, keep='first')
                idx = var.index.tolist()
                scores = var.to_numpy()
                order = np.argsort(-scores)
                order = order + 1

                ylabels = []
                ordering = []
                for x in args[1:]:
                    var2 = SDRank(self.config_path, self.log_path,
                                  x, dimension).calculateRank()
                    scores = var2['Result'].to_numpy()
                    val = np.argsort(-scores, kind='stable')
                    # Drop that column
                    var2.drop(['Result'], axis=1, inplace=True)
                    # Put whatever series you want in its place
                    var2['Result'] = val
                    var2 = var2['Result']

                    var2 = var2.loc[idx]
                    order2_list = var2[:].tolist()
                    order2 = [x+1 for x in order2_list]
                    order2 = np.asarray(order2)

                    # CREATE YLABEL AND ORDERING
                    ylabels.append(str(args[0]) + "-" + str(x))
                    order1_2 = np.subtract(order, order2)
                    order1_2 = np.absolute(order1_2)
                    ordering.append(order1_2)

                orders = np.stack((ordering), axis=0)
                xlabels = idx

                plt.rc('xtick', labelsize=8)
                plt.rc('ytick', labelsize=14)
                plt.figure(figsize=(22, 5))
                sns.heatmap(orders,
                            cmap='YlOrBr',
                            vmin=0,
                            xticklabels=xlabels,
                            yticklabels=ylabels,
                            annot=True,
                            square=True,
                            annot_kws={'fontsize': 8, 'fontweight': 'bold'})
                plt.yticks(rotation=0)
                plt.tick_params(
                    which='both',
                    bottom=False,
                    left=False,
                    labelbottom=False,
                    labeltop=True)
                return plt.tight_layout()

        elif dimension not in self.li:
            raise Exception("Dimension is not in set")

    def heatMap(self, rankset1: str, rankset2: str, dimension: str):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        dims = list(d.keys())

        if dimension in self.li:
            if dimension == "paretoAgg":
                data1 = MDRank(self.config_path, self.log_path,
                               rankset1).paretoAgg()
                data2 = MDRank(self.config_path, self.log_path,
                               rankset2).paretoAgg()

                data1_1 = data1['Solution']
                data1_1.replace('', np.nan, inplace=True)
                data1_1.dropna(inplace=True)
                data1_2 = data1['Dominated']
                data1_2.replace('', np.nan, inplace=True)
                data1_2.dropna(inplace=True)
                data1 = pd.concat([data1_1, data1_2],
                                  axis=0, ignore_index=True)

                data2_1 = data2['Solution']
                data2_1.replace('', np.nan, inplace=True)
                data2_1.dropna(inplace=True)
                data2_2 = data2['Dominated']
                data2_2.replace('', np.nan, inplace=True)
                data2_2.dropna(inplace=True)
                data2 = pd.concat([data2_1, data2_2],
                                  axis=0, ignore_index=True)

                data1 = data1[:10]
                order1 = list(data1.index)
                idx = data1.tolist()
                order1 = [x+1 for x in order1]

                data2 = pd.Series(data2.index.values, index=data2)
                data2 = data2.loc[idx]
                data2 = data2.tolist()
                order2 = [x+1 for x in data2]

                xlabels = idx
                ylabels = [str(rankset1), str(rankset2)]
                orders = np.stack((order1, order2), axis=0)

                plt.rc('xtick', labelsize=10)
                plt.rc('ytick', labelsize=14)
                plt.figure(figsize=(22, 5))
                sns.heatmap(orders,
                            cmap='YlOrBr',
                            vmin=0,
                            xticklabels=xlabels,
                            yticklabels=ylabels,
                            annot=True,
                            square=True,
                            annot_kws={'fontsize': 8, 'fontweight': 'bold'})
                plt.yticks(rotation=0)
                plt.tick_params(
                    which='both',
                    bottom=False,
                    left=False,
                    labelbottom=False,
                    labeltop=True)
                return plt.tight_layout()

            elif dimension == "paretoQ":
                data1 = MDRank(self.config_path, self.log_path,
                               rankset1).paretoQ()
                data2 = MDRank(self.config_path, self.log_path,
                               rankset2).paretoQ()

                data1_1 = data1['Solution']
                data1_1.replace('', np.nan, inplace=True)
                data1_1.dropna(inplace=True)
                data1_2 = data1['Dominated']
                data1_2.replace('', np.nan, inplace=True)
                data1_2.dropna(inplace=True)
                data1 = pd.concat([data1_1, data1_2],
                                  axis=0, ignore_index=True)

                data2_1 = data2['Solution']
                data2_1.replace('', np.nan, inplace=True)
                data2_1.dropna(inplace=True)
                data2_2 = data2['Dominated']
                data2_2.replace('', np.nan, inplace=True)
                data2_2.dropna(inplace=True)
                data2 = pd.concat([data2_1, data2_2],
                                  axis=0, ignore_index=True)

                data1 = data1[:10]
                order1 = list(data1.index)
                idx = data1.tolist()
                order1 = [x+1 for x in order1]

                data2 = pd.Series(data2.index.values, index=data2)
                data2 = data2.loc[idx]
                data2 = data2.tolist()
                order2 = [x+1 for x in data2]

                xlabels = idx
                ylabels = [str(rankset1), str(rankset2)]
                orders = np.stack((order1, order2), axis=0)

                plt.rc('xtick', labelsize=10)
                plt.rc('ytick', labelsize=14)
                plt.figure(figsize=(22, 5))
                sns.heatmap(orders,
                            cmap='YlOrBr',
                            vmin=0,
                            xticklabels=xlabels,
                            yticklabels=ylabels,
                            annot=True,
                            square=True,
                            annot_kws={'fontsize': 8, 'fontweight': 'bold'})
                plt.yticks(rotation=0)
                plt.tick_params(
                    which='both',
                    bottom=False,
                    left=False,
                    labelbottom=False,
                    labeltop=True)
                return plt.tight_layout()
            
            elif dimension == "RTA":
                var1 = RTA(self.config_path, self.log_path, rankset1).rta()
                var1 = var1['Rta_Scores']
                var1 = var1.nsmallest(n=10, keep='first')
                idx = var1.index.tolist()
                scores = var1.to_numpy()
                order1 = np.argsort(scores, kind='stable')
                order1 = order1+1

                var2 = RTA(self.config_path, self.log_path, rankset2).rta()
                scores = var2['Rta_Scores'].to_numpy()
                val = np.argsort(scores, kind='stable')
                # Drop that column
                var2.drop(['Rta_Scores'], axis=1, inplace=True)
                # Put whatever series you want in its place
                var2['Rta_Scores'] = val
                var2 = var2['Rta_Scores']

                var2 = var2.loc[idx]
                order2_list = var2[:].tolist()
                order2 = [x+1 for x in order2_list]
                order2 = np.asarray(order2)

                xlabels = var1.index.values.tolist()
                xlabels = xlabels[:10]
                ylabels = [str(rankset1), str(rankset2)]
                orders = np.stack((order1, order2), axis=0)

                plt.rc('xtick', labelsize=10)
                plt.rc('ytick', labelsize=14)
                plt.figure(figsize=(22, 5))
                sns.heatmap(orders,
                            cmap='YlOrBr',
                            vmin=0,
                            xticklabels=xlabels,
                            yticklabels=ylabels,
                            annot=True,
                            square=True,
                            annot_kws={'fontsize': 8, 'fontweight': 'bold'})
                plt.yticks(rotation=0)
                plt.tick_params(
                    which='both',
                    bottom=False,
                    left=False,
                    labelbottom=False,
                    labeltop=True)
                return plt.tight_layout()

            else:
                var1 = SDRank(self.config_path, self.log_path,
                              rankset1, dimension).calculateRank()
                var1 = var1['Result']
                var1 = var1.nlargest(n=10, keep='first')
                idx = var1.index.tolist()
                scores = var1.to_numpy()
                order1 = np.argsort(-scores, kind="stable")
                order1 = order1 + 1

                var2 = SDRank(self.config_path, self.log_path,
                              rankset2, dimension).calculateRank()
                scores = var2['Result'].to_numpy()
                val = np.argsort(-scores, kind='stable')
                # Drop that column
                var2.drop(['Result'], axis=1, inplace=True)
                # Put whatever series you want in its place
                var2['Result'] = val
                var2 = var2['Result']

                var2 = var2.loc[idx]
                order2_list = var2[:].tolist()
                order2 = [x+1 for x in order2_list]
                order2 = np.asarray(order2)

                xlabels = var1.index.values.tolist()
                xlabels = xlabels[:10]
                ylabels = [str(rankset1), str(rankset2)]
                orders = np.stack((order1, order2), axis=0)

                plt.rc('xtick', labelsize=10)
                plt.rc('ytick', labelsize=14)
                plt.figure(figsize=(22, 5))
                sns.heatmap(orders,
                            cmap='YlOrBr',
                            vmin=0,
                            xticklabels=xlabels,
                            yticklabels=ylabels,
                            annot=True,
                            square=True,
                            annot_kws={'fontsize': 8, 'fontweight': 'bold'})
                plt.yticks(rotation=0)
                plt.tick_params(
                    which='both',
                    bottom=False,
                    left=False,
                    labelbottom=False,
                    labeltop=True)
                return plt.tight_layout()

        elif dimension not in self.li:
            raise Exception("Dimension is not in set")



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '1.1.4'
DESCRIPTION = 'A library for performance analysis of sql based RDF graph processing systems'
LONG_DESCRIPTION = 'PAPyA simplifies the performance analysis of big data systems for processing large RDF graphs. This is an open-source library under an MIT license that will be a catalyst for designing new research prescriptive analysis techniques for big data applications'

# Setting up
setup(
    name="PAPyA",
    version=VERSION,
    author="Mohammed Ragab, Adam Satria Adidarma, Riccardo Tommasini",
    author_email="mohammed.ragab@ut.ee, adam.19051@mhs.its.ac.id, riccardo.tommasini@ut.ee",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=['matplotlib', 'notebook', 'PyYAML', 'pandas', 'scipy', 'plotly', 'seaborn'],
    keywords=['python', 'benchmarking', 'rdf systems', 'big data', 'apache spark'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


from config_loader import Loader
from joinTuple import joinTuple
import itertools
import time
import os.path
import os
import numpy as np
import pandas as pd


class FileReader(Loader, joinTuple):
    def __init__(self, config_path: str, log_path: str, size: str, sd: str, joined_string=None, deleted_config=None):
        super().__init__(config_path)
        self.log_path = log_path
        self.size = size
        self.sd = sd
        self.joined_string = joined_string
        self.deleted_config = deleted_config
    
    def deleteConfig(self):
        return self.deleted_config

    def file_reader(self):
        loader = Loader(self.config_path)
        data = loader.loader()
        d = data.get('dimensions')
        options = list(d.values())
        query = data.get('query')

        tuple = list(itertools.product(*options))
        self.joined_string = map(joinTuple.join_tuple_string, tuple)
        self.joined_string = list(self.joined_string)

        avg = []
        self.deleted_config = []
        time_to_wait = 10
        time_counter = 0
        for i in self.joined_string:
            try:
                pd.read_csv(f'{self.log_path}/{self.size}/{i}.txt',
                            sep=',', header=None)
            except FileNotFoundError:
                self.deleted_config.append(i)
                f = open(f"{self.log_path}/{self.size}/{i}.txt", "w+")
                f.write(",".join(str(0) for i in range(query)))
                f.close()
                while not os.path.exists(f"{self.log_path}/{self.size}/{i}.txt"):
                    time.sleep(1)
                    time_counter += 1
                    if time_counter > time_to_wait:
                        break

        for i in self.joined_string:
            df = pd.read_csv(f'{self.log_path}/{self.size}/{i}.txt',
                             sep=',', header=None)
            df = df.fillna(0)
            mean = df.mean(axis=0, numeric_only=True)
            avg.append(mean)

        df = pd.DataFrame(avg, index=self.joined_string)
        df = df.set_axis(["Q"+str(i+1) for i in range(query)], axis=1)
        df = df.replace(0, np.nan)
        max_value = df.max()
        max_value = max_value.apply(lambda x: x*(11/10))
        df = df.fillna(max_value)

        if self.sd != list(d.keys())[-1]:
            li = []
            config = list(df.index)
            d_list = list(d)
            key_slice = list(itertools.islice(
                d_list, d_list.index(self.sd)+1, None))
            number_of_options = [d[x]for x in key_slice]
            total_length = len(list(itertools.product(*number_of_options)))
            for counter in range(total_length):
                for x in range(int(len(config)/total_length)):
                    li.append(config[counter])
                    counter = counter+total_length
            df = df.reindex(index=li)

        if len(self.deleted_config) == 0:
            return df
        elif len(self.deleted_config) != 0:
            for i in self.deleted_config:
                os.remove(f"{self.log_path}/{self.size}/{i}.txt")

            return df

        # elif args != None:
        #     options = []
        #     store = []
        #     for arg in args:
        #         if isinstance(arg, str):
        #             if arg not in d.get(self.sd):
        #                 options.append(arg)
        #             elif arg in d.get(self.sd):
        #                 raise Exception(
        #                     "Options cannot be in the choosen dimension")
        #         else:
        #             pass
        #     for x in range(len(options)):
        #         c = fr'(?=.*\b{options[x]}\b)'
        #         store.append(c)
        #     command = "".join(store)
        #     df = df.loc[df.index.str.contains(command, regex=True)]
        #     return df



from operator import index, indexOf
import numpy as np


class kendallIndex:
    def __init__(self, var1: list, var2: list):
        self.var1 = var1
        self.var2 = var2

    def normalised_kendall_tau_distance(self):
        n = len(self.var1)
        assert len(self.var2) == n, "Both lists have to be of equal length"
        i, j = np.meshgrid(np.arange(n), np.arange(n))
        a = np.argsort(self.var1)
        b = np.argsort(self.var2)
        ndisordered = np.logical_or(np.logical_and(
            a[i] < a[j], b[i] > b[j]), np.logical_and(a[i] > a[j], b[i] < b[j])).sum()
        return ndisordered / (n * (n - 1))

    # ON DEVELOPMENT

    # def parr(self, input):
    #     return self.pairs(list(input))

    # def pairs(self, input):
    #     array = input[0]
    #     result = []
    #     for i in range(len(array)-1):
    #         for j in range(len(array)):
    #             j = i+1
    #             result.append(i)
    #             result.append(j)
    #     return result

    # def union(self, a, b):
    #     x = []
    #     check = lambda e,cb: x.append(e) if e not in x else cb = None if type(cb)

    # def check(self, e, cb):
    #     if cb and type(cb) == 'function':
    #         cb = None
    #     try:
    #         indexOf(self.x, e)
    #     except ValueError:
    #         self.x.append(e)

    # def my_rank(self):
    #     r = self.var1[0]
    #     rb = self.var2[0]
    #     pairs = self.parr(self.union(r, rb))
    #     print(pairs)
    #     count = 0
    #     c = len(pairs)

    #     def indexOf(r, e):
    #         if e in r:
    #             r.index(e)
    #         if e not in r:
    #             c

    #     for j in range(c):
    #         pair = pairs[j]
    #         if indexOf(r, pair[0]) == indexOf(r, pair[1]):
    #             if indexOf(rb, pair[0]) == indexOf(rb, pair[1]):
    #                 count = count
    #             else:
    #                 count = count + 1
    #         elif indexOf(r, pair[0]) > indexOf(r, pair[1]):
    #             if indexOf(rb, pair[0]) <= indexOf(rb, pair[1]):
    #                 count = count+1
    #             else:
    #                 count = count
    #         elif indexOf(r, pair[0]) < indexOf(r, pair[1]):
    #             if indexOf(rb, pair[0]) >= indexOf(rb, pair[1]):
    #                 count = count+1
    #             else:
    #                 count = count

    #     return count
